import type { Cookies } from "@sveltejs/kit";
import Cryptr from "./crypto";
import env from "$lib/env";
import type { Schema } from "zod";
import { encode, decode } from "msgpackr";

const cryptr = new Cryptr(env.COOKIE_SECRET);

export class SecureCookie<T> {
  constructor(
    protected cookies: Cookies,
    protected schema: Schema<T>,
    readonly name: string,
  ) {}

  read(): T | null {
    const cookie = this.cookies.get(this.name);
    if (!cookie) return null;

    try {
      const decryptedBuffer = cryptr.decrypt(cookie);
      const json = decode(decryptedBuffer);
      const data = this.schema.parse(json);
      return data;
    } catch {
      return null;
    }
  }

  write(data: T) {
    const compactJson = encode(data);
    const encryptedBuffer = cryptr.encrypt(compactJson);
    this.cookies.set(this.name, encryptedBuffer, { path: "/" });
  }
}
