import crypto from "crypto";

// Inspiré de https://github.com/MauriceButler/cryptr/blob/master/index.js
export default class Cryptr {
  algorithm = "aes-128-gcm" as const;
  encoding = "base64" as const;
  ivLength = 16;
  tagLength = 16;
  encryptedPosition = this.ivLength + this.tagLength;

  constructor(public secret: string) {}

  getKey() {
    return crypto
      .createHash("sha256")
      .update(this.secret)
      .digest()
      .subarray(0, 16);
  }

  encrypt(value: crypto.BinaryLike): string {
    const iv = crypto.randomBytes(this.ivLength);

    const key = this.getKey();

    const cipher = crypto.createCipheriv(this.algorithm, key, iv);
    const encrypted = Buffer.concat([cipher.update(value), cipher.final()]);

    const tag = cipher.getAuthTag();

    return Buffer.concat([iv, tag, encrypted]).toString(this.encoding);
  }

  decrypt(value: string): Buffer {
    const stringValue = Buffer.from(String(value), this.encoding);

    const iv = stringValue.subarray(0, this.ivLength);
    const tag = stringValue.subarray(this.ivLength, this.encryptedPosition);
    const encrypted = stringValue.subarray(this.encryptedPosition);

    const key = this.getKey();

    const decipher = crypto.createDecipheriv(this.algorithm, key, iv);

    decipher.setAuthTag(tag);

    return Buffer.concat([decipher.update(encrypted), decipher.final()]);
  }
}
