import type { Cookies } from "@sveltejs/kit";
import { SecureCookie } from "./cookie";
import { z } from "zod";

export enum GameStage {
  NEW,
  PLAYING,
  SOLUTION,
  NEXT,
  GAME_OVER,
}

export const gameStateSchema = z.object({
  history: z.array(z.string()),
  step: z.number(),
  score: z.number(),
  options: z.array(z.string()),
  stage: z.nativeEnum(GameStage),
  year: z.number(),
  maxYear: z.number().optional(),
  label: z.string(),
  solution: z.string(),
  timestamp: z.number(),
});

export type GameState = z.infer<typeof gameStateSchema>;

export class Game {
  state: GameState;
  protected cookie_name = "quiestce-quiz";
  protected cookie: SecureCookie<GameState>;

  constructor(protected cookies: Cookies) {
    this.cookie = new SecureCookie(cookies, gameStateSchema, this.cookie_name);
    this.state = this.cookie.read() ?? this.defaultState();
    // console.log('LOAD:', GameStage[this.state.stage], this.state);
  }

  protected defaultState(): GameState {
    return {
      stage: GameStage.NEW,
      history: [],
      step: 0,
      score: 0,
      year: 0,
      options: [],
      solution: "",
      label: "?",
      timestamp: 0,
    };
  }

  save() {
    // console.log('SAVE:', GameStage[this.state.stage], this.state);
    this.state.timestamp = Date.now();
    this.cookie.write(this.state);
  }

  reset() {
    this.state = this.defaultState();
    this.save();
  }
}
